@@ -47,6 +47,7 @@ from terraform_compliance.steps.then.it_must_have_reference_address_referenced i
 from terraform_compliance.steps.then.its_key_condition_be_value import its_key_condition_be_value
 from terraform_compliance.steps.then.interpolations import i_flatten_everything_found
 from terraform_compliance.steps.then.it_must_be_in import it_must_be_in
+from terraform_compliance.steps.then.it_must_not_be_in import it_must_not_be_in
 from terraform_compliance.steps.then.its_value_condition_be_null import its_value_condition_be_null
 
 # {name} is checked for startswith("resource that supports "). The @given decorator for that documented case is not needed
@@ -61,8 +62,14 @@ def wrapper(_step_obj, name, type_name='resource', _terraform_config=world):
 @when(u'its {key:PROPERTY} metadata contains {value:PROPERTY}')
 @when(u'its {key:PROPERTY} metadata includes {value:PROPERTY}')
 def wrapper(_step_obj, key, value):
-    return its_key_metadata_has_something(_step_obj, key, value)
+    return its_key_metadata_has_something(_step_obj, key, value, regex_match=False)
 
+@when(u'its {key:PROPERTY} metadata has "{search_regex}" regex')
+@when(u'its {key:PROPERTY} metadata is "{search_regex}" regex')
+@when(u'its {key:PROPERTY} metadata contains "{search_regex}" regex')
+@when(u'its {key:PROPERTY} metadata includes "{search_regex}" regex')
+def wrapper(_step_obj, key, search_regex):
+    return its_key_metadata_has_something(_step_obj, key, search_regex, regex_match=True)
 
 @when(u'its {key:PROPERTY} metadata has not {value:PROPERTY}')
 @when(u'its {key:PROPERTY} metadata is not {value:PROPERTY}')
@@ -72,6 +79,13 @@ def wrapper(_step_obj, key, value):
 def wrapper(_step_obj, key, value):
     return its_key_metadata_has_something(_step_obj, key, value, has_step=False)
 
+@when(u'its {key:PROPERTY} metadata has not "{search_regex}" regex')
+@when(u'its {key:PROPERTY} metadata is not "{search_regex}" regex')
+@when(u'its {key:PROPERTY} metadata does not have "{search_regex}" regex')
+@when(u'its {key:PROPERTY} metadata does not contain "{search_regex}" regex')
+@when(u'its {key:PROPERTY} metadata does not include "{search_regex}" regex')
+def wrapper(_step_obj, key, search_regex):
+    return its_key_metadata_has_something(_step_obj, key, search_regex, has_step=False, regex_match=True)
 
 @when(u'its {key:PROPERTY} is {value:PROPERTY}')
 @when(u'its {key:PROPERTY} has {value:PROPERTY}')
@@ -83,7 +97,19 @@ def wrapper(_step_obj, key, value):
 @when(u'its {address:PROPERTY} {key:PROPERTY} contains {value:PROPERTY}')
 @when(u'its {key:PROPERTY} includes an entry where {value:PROPERTY} is {dict_value:PROPERTY}')
 def wrapper(_step_obj, key, value, dict_value=None, address=Null):
-    return its_key_is_value(_step_obj, key, value, dict_value, address)
+    return its_key_is_value(_step_obj, key, value, dict_value, address, regex_match=False)
+
+@when(u'its {key:PROPERTY} is "{search_regex}" regex')
+@when(u'its {key:PROPERTY} has "{search_regex}" regex')
+@when(u'its {key:PROPERTY} includes "{search_regex}" regex')
+@when(u'its {key:PROPERTY} contains "{search_regex}" regex')
+@when(u'its {address:PROPERTY} {key:PROPERTY} is "{search_regex}" regex')
+@when(u'its {address:PROPERTY} {key:PROPERTY} has "{search_regex}" regex')
+@when(u'its {address:PROPERTY} {key:PROPERTY} includes "{search_regex}" regex')
+@when(u'its {address:PROPERTY} {key:PROPERTY} contains "{search_regex}" regex')
+@when(u'its {key:PROPERTY} includes an entry where {value:PROPERTY} is "{dict_value}" regex')
+def wrapper(_step_obj, key, search_regex, dict_value=None, address=Null):
+    return its_key_is_value(_step_obj, key, search_regex, dict_value, address, regex_match=True)
 
 
 @when(u'its {key:PROPERTY} is not {value:PROPERTY}')
@@ -96,8 +122,19 @@ def wrapper(_step_obj, key, value, dict_value=None, address=Null):
 @when(u'its {address:PROPERTY} {key:PROPERTY} does not contain {value:PROPERTY}')
 @when(u'its {key:PROPERTY} does not include an entry where {value:PROPERTY} is {dict_value:PROPERTY}')
 def wrapper(_step_obj, key, value, dict_value=None, address=Null):
-    return its_key_is_not_value(_step_obj, key, value, dict_value, address)
+    return its_key_is_not_value(_step_obj, key, value, dict_value, address, regex_match=False)
 
+@when(u'its {key:PROPERTY} is not "{search_regex}" regex')
+@when(u'its {key:PROPERTY} has not "{search_regex}" regex')
+@when(u'its {key:PROPERTY} does not include "{search_regex}" regex')
+@when(u'its {key:PROPERTY} does not contain "{search_regex}" regex')
+@when(u'its {address:PROPERTY} {key:PROPERTY} is not "{search_regex}" regex')
+@when(u'its {address:PROPERTY} {key:PROPERTY} has not "{search_regex}" regex')
+@when(u'its {address:PROPERTY} {key:PROPERTY} does not include "{search_regex}" regex')
+@when(u'its {address:PROPERTY} {key:PROPERTY} does not contain "{search_regex}" regex')
+@when(u'its {key:PROPERTY} does not include an entry where {value:PROPERTY} is "{dict_value}" regex')
+def wrapper(_step_obj, key, search_regex, dict_value=None, address=Null):
+    return its_key_is_not_value(_step_obj, key, search_regex, dict_value, address, regex_match=True)
 
 @when(u'it contain {something:PROPERTY}') # This is just here for not breaking backward compatibility. I know its wrong.
 @when(u'it contains {something:PROPERTY}')
@@ -244,4 +281,14 @@ def wrapper(_step_obj, haystack):
 @then('it must be a superset of {haystack:ANY}')
 @then('it must cover {haystack:ANY}')
 def wrapper(_step_obj, haystack):
-    return it_must_be_in(_step_obj, haystack, cover=True)
\ No newline at end of file
+    return it_must_be_in(_step_obj, haystack, cover=True)
+
+@then('it must not be a subset of {haystack:ANY}')
+@then('it must not be in {haystack:ANY}')
+def wrapper(_step_obj, haystack):
+    return it_must_not_be_in(_step_obj, haystack)
+
+@then('it must not be a superset of {haystack:ANY}')
+@then('it must not cover {haystack:ANY}')
+def wrapper(_step_obj, haystack):
+    return it_must_not_be_in(_step_obj, haystack, cover=True)
\ No newline at end of file
